@@ -17,11 +17,15 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <stdlib.h>
 #include <string.h>
+
+#undef GTK_DISABLE_DEPRECATED
+
 #include <gtk/gtk.h>
+
 #include "gailtoplevel.h"
 
 static void             gail_toplevel_class_init        (GailToplevelClass      *klass);
@@ -90,7 +94,7 @@ gail_toplevel_init (GailToplevel *toplevel)
       window = GTK_WINDOW (l->data);
       widget = GTK_WIDGET (window);
       if (!window || 
-          !GTK_WIDGET_VISIBLE (widget) ||
+          !gtk_widget_get_visible (widget) ||
           is_attached_menu_window (widget) ||
           GTK_WIDGET (window)->parent ||
           GTK_IS_PLUG (window))
@@ -110,7 +114,7 @@ gail_toplevel_init (GailToplevel *toplevel)
         }
     }
 
-  gtk_type_class (GTK_TYPE_WINDOW);
+  g_type_class_ref (GTK_TYPE_WINDOW);
 
   signal_id  = g_signal_lookup ("show", GTK_TYPE_WINDOW);
   g_signal_add_emission_hook (signal_id, 0,
